import os
from typing import List, Dict, Any

import yaml
from jinja2 import Environment, FileSystemLoader, select_autoescape, StrictUndefined

TEMPLATE_PATH = os.path.join(os.path.dirname(__file__), "templates")


def render_config(
    helm_config_path: str, output_path: str, params: Dict[str, str]
) -> None:
    # get helm data
    configmaps = get_helm_data(helm_config_path, file_type="configmap")
    env = get_helm_data(helm_config_path, file_type="registrar-deployment", index=True)

    # extract envs
    envs = extract_env_vars(env)

    # merge envs and params
    merged_params = {**params, **envs}

    # prepare the configs
    mapping = prepare_configs(configmaps)

    # create folder with name of slug at path
    slug = params["slug"]
    destination = prepare_directory(output_path, slug)

    # create necessary configs and store at appropriate path
    create_configurations(destination, mapping)

    # fill remaining templates and store at appropriate path
    render_templates(destination, merged_params)


def get_helm_data(helm_config_path: str, file_type: str, index: bool = False) -> Any:
    if not os.path.isdir(helm_config_path):
        raise ValueError(f"{helm_config_path} is not a directory")

    files = [
        os.path.join(helm_config_path, f)
        for f in os.listdir(helm_config_path)
        if os.path.isfile(os.path.join(helm_config_path, f)) and file_type in f
    ]

    if index:
        return files[0]

    return files


def prepare_configs(configmaps: List[str]) -> Dict[str, str]:
    file_config_mapping = {}
    for configmap in configmaps:
        with open(configmap, "r", encoding="utf-8") as cfg:
            config = yaml.load(cfg, Loader=yaml.SafeLoader)
        config = config["data"]
        if config:
            file_name = list(config.keys())[0]
            config = config[file_name]
        else:
            continue

        file_config_mapping[file_name] = config

    return file_config_mapping


def prepare_directory(output_path: str, slug: str) -> str:
    if os.path.isdir(output_path):
        destination = os.path.join(output_path, slug)
        try:
            os.mkdir(destination)
        except FileExistsError:
            pass
    else:
        raise ValueError(f"{output_path} is not a directory")

    return destination


def create_configurations(destination: str, mapping: Dict[str, str]) -> None:
    destination = os.path.join(destination, "config")
    try:
        os.mkdir(destination)
    except FileExistsError:
        pass

    for filename, config in mapping.items():
        config_path = os.path.join(destination, filename)
        with open(config_path, "w", encoding="utf-8") as cfg:
            cfg.write(config)


def render_templates(destination: str, params: Dict[str, str]) -> None:
    jinja_env = Environment(
        loader=FileSystemLoader(TEMPLATE_PATH),
        autoescape=select_autoescape(),
        undefined=StrictUndefined,
    )

    templates = os.listdir(TEMPLATE_PATH)
    for template in templates:
        tpl = jinja_env.get_template(template)
        rendered_template = tpl.render(**params)
        template_destination = (
            os.path.join(destination, template)
            if "docker" in template
            else os.path.join(destination, "config", template)
        )
        with open(template_destination, "w", encoding="utf-8") as tpl_file:
            tpl_file.write(rendered_template)


def extract_env_vars(env: str) -> Dict[str, str]:
    envs = {}
    with open(env, "r", encoding="utf-8") as cfg:
        config = yaml.load(cfg, Loader=yaml.SafeLoader)
        env_vars = config["spec"]["template"]["spec"]["containers"][0]["env"]
        for var in env_vars:
            envs[var["name"]] = var["value"]

    return envs
